<?php

namespace App\Http\Controllers;

use App\Http\Requests\CreateBranchRequest;
use App\Http\Requests\UpdateBranchProductsRequest;
use App\Http\Requests\UpdateBranchRequest;
use App\Models\Branch;
use App\Models\BranchProduct;
use App\Models\Category;
use App\Models\Limitation;
use App\Models\Product;
use App\Services\Branch\AllBranches;
use App\Services\Branch\CreateBranch;
use App\Services\Branch\DeleteBranch;
use App\Services\Branch\ReadBranch;
use App\Services\Branch\UpdateBranch;
use App\Services\Branch\UpdateBranchProducts;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Validation\Rule;

class BranchController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth:sanctum')->except('index');
    }

    /**
     * @param AllBranches $allBranches
     * @return JsonResponse
     */
    public function index(AllBranches $allBranches): JsonResponse
    {
        $response = $allBranches->all();
        return jsonResponse($response);
    }

    /**
     * @param CreateBranchRequest $request
     * @param CreateBranch $branch
     * @return JsonResponse
     */
    public function store(CreateBranchRequest $request, CreateBranch $branch): JsonResponse
    {
        $response = $branch->create($request);
        return jsonResponse($response);
    }

    /**
     * @param ReadBranch $branch
     * @param $id
     * @return JsonResponse
     */
    public function show(ReadBranch $branch, $id): JsonResponse
    {
        $response = $branch->find($id);
        return jsonResponse($response);
    }

    /**
     * @param UpdateBranchRequest $request
     * @param UpdateBranch $branch
     * @param $id
     * @return JsonResponse
     */
    public function update(UpdateBranchRequest $request, UpdateBranch $branch, $id): JsonResponse
    {
        $response = $branch->update($request, $id);
        return jsonResponse($response);
    }

    /**
     * @param UpdateBranchProductsRequest $request
     * @param UpdateBranchProducts $branchProducts
     * @param $id
     * @return JsonResponse
     */
    public function updateProducts(UpdateBranchProductsRequest $request, UpdateBranchProducts $branchProducts, $id): JsonResponse
    {
        $response = $branchProducts->update($request, $id);
        return jsonResponse($response);
    }

    /**
     * @param DeleteBranch $branch
     * @param $id
     * @return JsonResponse
     */
    public function destroy(DeleteBranch $branch, $id): JsonResponse
    {
        $response = $branch->delete($id);
        return jsonResponse($response);
    }

    /**
     * @return JsonResponse
     */
    public function mainProducts(): JsonResponse
    {
        $products = Product::query()->whereNull('deleted_at')->get();
        $storeProducts = [];
        foreach ($products as $product) {
            if (!$product->isCafe()) {
                array_push($storeProducts, $product);
            }
        }
        return jsonResponse([
            'success' => true,
            'products' => $storeProducts
        ]);
    }

    /**
     * @param $product_id
     * @param $branch_id
     * @return JsonResponse
     */
    public function removeFromBranch($product_id, $branch_id): JsonResponse
    {
        BranchProduct::query()
            ->where('product_id', $product_id)
            ->where('branch_id', $branch_id)
            ->delete();
        return jsonResponse([
            'success' => true
        ]);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     */
    public function addProducts(Request $request): JsonResponse
    {
        $data = $request->validate([
            'branch_id' => 'required',
            'product_ids' => 'required'
        ]);
        foreach ($data['product_ids'] as $id) {
            BranchProduct::query()->create([
                'product_id' => $id,
                'branch_id' => $data['branch_id']
            ]);
        }
        return jsonResponse([
            'success' => true
        ]);
    }

    /**
     * @param Request $request
     * @param $branchId
     * @return JsonResponse
     */
    public function changeWorkHours(Request $request, $branchId): JsonResponse
    {
        $branch = findBranch($branchId);
        if (!$branch) {
            return jsonResponse([
                'success' => false,
                'message' => 'شعبه مورد نظر یافت نشد'
            ]);
        }
        $data = $request->validate([
            'end_hour' => ['required', 'min:1', 'max:24'],
            'start_hour' => ['required', 'min:1', 'max:24'],
        ]);
        return jsonResponse($this->handleHours($branch, $data['start_hour'], $data['end_hour']));
    }

    /**
     * @param $branch
     * @param $start_hour
     * @param $end_hour
     * @return array
     */
    protected function handleHours($branch, $start_hour, $end_hour): array
    {
        if ($start_hour == $end_hour) {
            return [
                'success' => false,
                'message' => 'ساعت شروع به کار و پایان کار نمیتوانند با هم یکسان باشند'
            ];
        }
        if ($start_hour > $end_hour) {
            return [
                'success' => false,
                'message' => 'ساعت شروع به کار نمیتواند از ساعت پایان کار زود تر باشد'
            ];
        }
        $branch->update([
            'start_hour' => $start_hour,
            'end_hour' => $end_hour
        ]);
        return [
            'success' => true
        ];
    }

    /**
     * @return JsonResponse
     */
    public function cafeCategories(): JsonResponse
    {
        $categories = Category::query()
            ->where('category_type', Product::class)
            ->where('type', 'CAFE')
            ->get();
        return jsonResponse([
            'success' => true,
            'categories' => $categories
        ]);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     */
    public function updateQuantity(Request $request): JsonResponse
    {
        $data = $request->validate([
            'branch_id' => ['required', Rule::exists('branches', 'id')]
        ]);
        $categories = Category::query()
            ->where('category_type', Product::class)
            ->where('type', 'CAFE')
            ->get();
        Limitation::query()
            ->where('branch_id', $data['branch_id'])
            ->where('quantity', '<=', 100)
            ->update(['quantity' => 100]);
        DB::transaction(function () use ($categories, $data) {
            foreach ($categories as $category) {
                $values = [
                    'branch_id' => $data['branch_id'],
                    'category_id' => $category->id,
                    'quantity' => 100
                ];
                Limitation::query()->updateOrCreate($values, $values);
            }
        });
        return jsonResponse([
            'success' => true
        ]);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     */
    public function categoryQuantity(Request $request): JsonResponse
    {
        $data = $request->validate([
            'limitation_id' => ['required', Rule::exists('limitations', 'id')],
            'quantity' => ['required', 'integer', 'min:0']
        ]);
        $limitation = Limitation::query()->find($data['limitation_id']);
        $limitation->update([
            'quantity' => $data['quantity']
        ]);
        return jsonResponse([
            'success' => true,
        ]);
    }

    /**
     * @param Request $request
     * @param $id
     * @return JsonResponse
     */
    public function excuse(Request $request, $id): JsonResponse
    {
        $data = $request->validate([
            'excuse' => 'required'
        ]);
        $branch = Branch::query()->find($id);
        if (!$branch) {
            return jsonResponse([
                'success' => false
            ]);
        }
        $branch->update([
            'excuse' => $data['excuse']
        ]);
        return jsonResponse([
            'success' => true
        ]);
    }

    /**
     * @param $id
     * @return JsonResponse
     */
    public function branchProducts($id): JsonResponse
    {
        $branch = Branch::query()->find($id);
        if (!$branch) {
            return jsonResponse([
                'success' => false
            ]);
        }
        $products = $branch->products()->withPivot('capacity')->get();
        $mainProducts = [];
        foreach ($products as $product) {
            $name = $product->name;
            $capacity = $product->pivot->capacity;
            $limit = Limitation::query()
                ->where('category_id', $product->category_id)
                ->where('branch_id', $branch->id)
                ->first();
            $category_title = Category::query()->find($product->category_id)->title;
            $mainProducts[] = [
                'name' => $name,
                'capacity' => $capacity,
                'category_limit' => $this->remainingCategoryLimit($branch, $product),
                'product_id' => $product->id,
                'limitation_id' => $limit->id,
                'category_title' => $category_title
            ];
        }
        return jsonResponse([
            'success' => true,
            'products' => $mainProducts
        ]);
    }

    protected function remainingCategoryLimit($branch, $product)
    {
        $limitation = Limitation::query()
            ->where('category_id', $product->category_id)
            ->where('branch_id', $branch->id)
            ->first();
        $lim_quantity = $limitation->quantity ?? 0;
        $products = Product::query()->where('category_id', $product->category_id)->get();
        $sum = 0;
        foreach ($products as $item) {
            $sum += BranchProduct::query()
                ->where('branch_id', $branch->id)
                ->where('product_id', $item->id)
                ->first()
                ->capacity ?? 0;
        }
        return $lim_quantity - $sum;

    }

    /**
     * @param Request $request
     * @return JsonResponse
     */
    public function handleProductCapacity(Request $request): JsonResponse
    {
        $data = $request->validate([
            'branch_id' => 'required',
            'product_id' => 'required',
            'capacity' => 'required'
        ]);
        $branch = Branch::query()->find($data['branch_id']);
        if (!$branch) {
            return jsonResponse([
                'success' => false,
                'message' => 'شعبه مورد نظر یافت نشد.'
            ]);
        }
        $product = Product::query()->find($data['product_id']);
        if (!$product) {
            return jsonResponse([
                'success' => false,
                'message' => 'محصول مورد نظر یافت نشد.'
            ]);
        }
        if ($this->remainingCategoryLimit($branch, $product) - $data['capacity'] < 0) {
            return jsonResponse([
                'success' => false,
                'message' => 'عدد وارد شده از تعداد باقی مانده کل بیشتر است.'
            ]);
        }
        BranchProduct::query()->where('product_id', $product->id)->where('branch_id', $branch->id)->update([
            'capacity' => $data['capacity']
        ]);
        return jsonResponse([
            'success' => true
        ]);

    }
}
