<?php

namespace App\Http\Controllers;

use App\Codes\CodeGenerator;
use App\Exports\OrderExport;
use App\Http\Requests\CreateOrderRequest;
use App\Models\Order;
use App\Services\Order\AllOrders;
use App\Services\Order\AllTheOrders;
use App\Services\Order\ChangeStatus;
use App\Services\Order\CreateOrder;
use App\Services\Order\DeleteOrder;
use App\Services\Order\FindByTrackingCode;
use App\Services\Order\ReadFullOrder;
use App\Services\Order\ReadOrder;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;
use Maatwebsite\Excel\Facades\Excel;
use Symfony\Component\HttpFoundation\BinaryFileResponse;

class OrderController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth:sanctum');
        $this->middleware('branch')->only(['index', 'store', 'all', 'filter']);
        $this->middleware('work-time')->only('store');
    }

    /**
     * @param Request $request
     * @param AllOrders $orders
     * @return JsonResponse
     */
    public function index(Request $request, AllOrders $orders): JsonResponse
    {
        $response = $orders->all($request);
        return jsonResponse($response);
    }

    /**
     * @param CreateOrder $order
     * @param CreateOrderRequest $request
     * @param CodeGenerator $generator
     * @return JsonResponse
     */
    public function store(CreateOrder $order, CreateOrderRequest $request, CodeGenerator $generator): JsonResponse
    {
        $response = $order->save($request, $generator);
        return jsonResponse($response);
    }

    /**
     * @param Request $request
     * @param ReadOrder $order
     * @param $id
     * @return JsonResponse
     */
    public function show(Request $request, ReadOrder $order, $id): JsonResponse
    {
        $response = $order->read($request, $id);
        return jsonResponse($response);
    }

    /**
     * @param DeleteOrder $order
     * @param Request $request
     * @param $id
     * @return JsonResponse
     */
    public function destroy(DeleteOrder $order, Request $request, $id): JsonResponse
    {
        $response = $order->delete($request, $id);
        return jsonResponse($response);
    }

    /**
     * @param AllTheOrders $orders
     * @return JsonResponse
     */
    public function all(AllTheOrders $orders): JsonResponse
    {
        $response = $orders->all();
        return jsonResponse($response);
    }

    /**
     * @param ChangeStatus $status
     * @param $id
     * @return JsonResponse
     */
    public function changeStatus(ChangeStatus $status, $id): JsonResponse
    {
        $response = $status->change($id);
        return jsonResponse($response);
    }

    /**
     * @param ReadFullOrder $order
     * @param $id
     * @return JsonResponse
     */
    public function readOrder(ReadFullOrder $order, $id): JsonResponse
    {
        $response = $order->read($id);
        return jsonResponse($response);
    }

    /**
     * @param Request $request
     * @param AllTheOrders $orders
     * @return JsonResponse
     */
    public function filter(Request $request, AllTheOrders $orders): JsonResponse
    {
        $response = $orders->filter($request);
        return jsonResponse($response);
    }

    /**
     * @param Request $request
     * @param FindByTrackingCode $code
     * @return JsonResponse
     */
    public function findByTrackingCode(Request $request, FindByTrackingCode $code): JsonResponse
    {
        return jsonResponse($code->find($request));
    }

    /**
     * @param Request $request
     * @return JsonResponse
     */
    public function acceptOrder(Request $request): JsonResponse
    {
        $data = $request->validate([
            'tracking_code' => ['required', Rule::exists('orders', 'tracking_code')]
        ]);
        $order = Order::query()->where('tracking_code', $data['tracking_code'])->first();
        $order->update([
            'active' => false
        ]);
        return jsonResponse([
            'success' => true
        ]);
    }

    /**
     * @return BinaryFileResponse
     */
    public function export(): BinaryFileResponse
    {
        return Excel::download(new OrderExport(), 'cafe-orders.xlsx');
    }

    /**
     * @return BinaryFileResponse
     */
    public function exportStoreOrders(): BinaryFileResponse
    {
        return Excel::download(new OrderExport(), 'store-orders.xlsx');
    }

    /**
     * @return JsonResponse
     */
    public function cafeOrders(): JsonResponse
    {
        $service = new AllTheOrders();
        return jsonResponse($service->cafeOrders());
    }


    /**
     * @return JsonResponse
     */
    public function storeOrders(): JsonResponse
    {
        $service = new AllTheOrders();
        $response = $service->storeOrders();
        return jsonResponse($response);
    }
}
