<?php


namespace App\Http\Controllers;


use App\Models\Association;
use App\Models\BranchProduct;
use App\Models\Category;
use App\Models\CV;
use App\Models\Limitation;
use App\Models\Order;
use App\Models\Promotion;
use App\Models\Support;
use App\Models\Transaction;
use App\Models\User;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Http;
use Kavenegar\Exceptions\ApiException;
use Kavenegar\KavenegarApi;
use Shetabit\Multipay\Exceptions\PurchaseFailedException;
use Shetabit\Payment\Facade\Payment;

class PageController extends Controller
{
    public function terms()
    {
        $response = Http::get('https://api.emza-group.ir/public/api/terms');
        $terms = $response->json()['terms']['body'];
        return view('terms', compact('terms'));
    }

    public function paymentResult(Request $request)
    {
        $authority = $request->Authority;
        $response = $this->verify($authority);

        $message = $response['message'];
        $transaction = Transaction::query()->where('transaction_id', $authority)->first();
        $user = $transaction->user;
        if ($user->credit != null) {
            $user->credit->increment('value', $transaction->amount / 50000);
        } else {
            $userTransactions = $user->transactions;
            $value = 0;
            foreach ($userTransactions as $item) {
                $value += $item->amount;
            }
            $user->credit()->create([
                'value' => ($value / 50000)
            ]);
        }
        $device = $transaction->device;
        $specified_device = $device == 1 ? 'website' : 'mobile';
        return view('payment', compact('message', 'specified_device'));
    }

    public function verify($tx_id)
    {
        $transaction = Transaction::query()->where('transaction_id', $tx_id)->first();
        if (!$transaction) {
            return [
                'success' => false,
                'message' => 'تراکنش یافت نشد'
            ];
        }
        $transaction_id = $transaction->transaction_id;
        $amount = $transaction->amount;
        try {
            $receipt = Payment::amount($amount)->transactionId($transaction_id)->verify();
            $transaction->ref_id = $receipt->getReferenceId();
            $transaction->status = 'paid';
            $transaction->save();
            $user = User::query()->find($transaction->user_id);
            $order = $user->orders()->get()->last();
            $cartItems = $user->cartItems()->get();
            foreach ($cartItems as $item) {
                $order->orderItems()->create([
                    'product' => $item->product,
                    'price' => $item->price,
                    'quantity' => $item->quantity,
                    'sentence' => $item->sentence,
                    'frame' => $item->frame,
                    'to' => $item->to,
                    'from' => $item->from,
                    'size' => $item->size,
                ]);
            }
            $this->handleCategoryCapacities($user);
            $user->cartItems()->delete();
            $service = new KavenegarApi(config('kavenegar.apikey'));
            $service->VerifyLookup($user->getAttribute('phone_number'), $user->getAttribute('firstname'), null, null, 'ORDERCREATED');
            if ($order->promotion_code != null || $order->promotion_code != '') {
                Promotion::query()->where('promotion_code', $order->promotion_code)->delete();
            }
            return [
                'success' => true,
                'receipt' => $transaction->ref_id,
                'message' => 'تراکنش موفقیت آمیز بود'
            ];
        } catch (PurchaseFailedException $exception) {
            $transaction->status = 'failed';
            $transaction->save();
            return [
                'success' => true,
                'message' => $exception->getMessage()
            ];
        } catch (ApiException | Exception $exception) {
            return [
                'success' => true,
                'message' => $exception->getMessage()
            ];
        }
    }

    protected function handleCategoryCapacities($user)
    {
        $cartItems = $user->cartItems;
        foreach ($cartItems as $item) {
            $product = $item->product();
            $category = Category::query()->find($product->category_id);
            $limitation = Limitation::query()
                ->where('branch_id', $item->branch_id)
                ->where('category_id', $category->getAttribute('id'))
                ->first();
            $branchProduct = BranchProduct::query()
                ->where('branch_id', $item->branch_id)
                ->where('product_id', $product->id)
                ->first();
            if (exists($limitation)) {
                $limitation->decrement('quantity', $item->quantity);
            }
            if (exists($branchProduct)) {
                $branchProduct->decrement('capacity', $item->quantity);
            }
        }
    }

    public function init()
    {
        $cafeOrders = Order::query()
            ->whereHas('user')
            ->whereHas('transaction', function ($q) {
                $q->where('status', 'paid');
            })
            ->whereHas('orderItems.mainProduct.category', function ($q) {
                $q->where('type', 'CAFE');
            })
            ->where('active', true)
            ->count();
        $storeOrders = Order::query()
            ->whereHas('user')
            ->whereHas('transaction', function ($q) {
                $q->where('status', 'paid');
            })
            ->whereHas('orderItems.mainProduct.category', function ($q) {
                $q->where('type', 'STORE');
            })
            ->where('active', true)
            ->count();
        $newAssociations = Association::query()->where('status', 'PENDING')->count();
        $newCVs = CV::query()->where('status', 'PENDING')->count();
        $newSupports = Support::query()->where('status', 'PENDING')->count();
        return jsonResponse([
            'success' => true,
            'store_orders' => $storeOrders,
            'cafe_orders' => $cafeOrders,
            'associations' => $newAssociations,
            'cvs' => $newCVs,
            'supportItems' => $newSupports
        ]);
    }
}
