<?php

namespace App\Http\Controllers;

use App\Http\Requests\CreateProductRequest;
use App\Http\Requests\CreateStoreProductRequest;
use App\Http\Requests\UpdateStoreProductRequest;
use App\Models\Admin;
use App\Models\Product;
use App\Services\Product\AllProducts;
use App\Services\Product\AllStoreProducts;
use App\Services\Product\CreateProduct;
use App\Services\Product\CreateStoreProduct;
use App\Services\Product\DeleteProduct;
use App\Services\Product\ProductCategories;
use App\Services\Product\ReadProduct;
use App\Services\Product\StoreCategories;
use App\Services\Product\UpdateProduct;
use App\Services\Product\UpdateStoreProduct;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;

class ProductController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth:sanctum')->except(['show', 'storeProducts']);
        if (!(\request()->user() instanceof Admin)) {
            $this->middleware('branch')->only('index');
        }
    }

    /**
     * @param AllProducts $products
     * @param Request $request
     * @return JsonResponse
     */
    public function index(AllProducts $products, Request $request): JsonResponse
    {
        $response = $products->all($request);
        return jsonResponse($response);
    }

    /**
     * @param ReadProduct $product
     * @param $id
     * @return JsonResponse
     */
    public function show(ReadProduct $product, $id): JsonResponse
    {
        $response = $product->read($id);
        return jsonResponse($response);
    }

    /**
     * @param CreateProductRequest $request
     * @param CreateProduct $product
     * @return JsonResponse
     */
    public function store(CreateProductRequest $request, CreateProduct $product): JsonResponse
    {
        $response = $product->save($request);
        return jsonResponse($response);
    }

    /**
     * @param Request $request
     * @param UpdateProduct $product
     * @param $id
     * @return JsonResponse
     */
    public function update(Request $request, UpdateProduct $product, $id): JsonResponse
    {
        $response = $product->update($request, $id);
        return jsonResponse($response);
    }

    /**
     * @param $id
     * @param DeleteProduct $product
     * @return JsonResponse
     */
    public function destroy($id, DeleteProduct $product): JsonResponse
    {
        $response = $product->delete($id);
        return jsonResponse($response);
    }

    /**
     * @param ProductCategories $categories
     * @return JsonResponse
     */
    public function product_categories(ProductCategories $categories): JsonResponse
    {
        $response = $categories->show();
        return jsonResponse($response);
    }

    /**
     * @param Request $request
     * @param AllProducts $products
     * @return JsonResponse
     */
    public function filter(Request $request, AllProducts $products): JsonResponse
    {
        $response = $products->filter($request);
        return jsonResponse($response);
    }

    /**
     * @param AllStoreProducts $products
     * @return JsonResponse
     */
    public function storeProducts(AllStoreProducts $products): JsonResponse
    {
        return jsonResponse($products->all());
    }

    /**
     * @param StoreCategories $categories
     * @return JsonResponse
     */
    public function storeCategories(StoreCategories $categories): JsonResponse
    {
        return jsonResponse($categories->all());
    }

    /**
     * @param CreateStoreProductRequest $request
     * @param CreateStoreProduct $product
     * @return JsonResponse
     * @throws Exception
     */
    public function createStoreProduct(CreateStoreProductRequest $request, CreateStoreProduct $product): JsonResponse
    {
        return jsonResponse($product->create($request));
    }

    /**
     * @param UpdateStoreProductRequest $request
     * @param UpdateStoreProduct $product
     * @param $id
     * @return JsonResponse
     */
    public function updateStoreProduct(UpdateStoreProductRequest $request, UpdateStoreProduct $product, $id): JsonResponse
    {
        return jsonResponse($product->update($request, $id));
    }

    /**
     * @param $id
     * @return JsonResponse
     */
    public function togglePrint($id): JsonResponse
    {
        $product = Product::query()->find($id);
        if (!$product) {
            return jsonResponse([
                'success' => false
            ]);
        }
        $product->update([
            'printable' => !$product->getAttribute('printable')
        ]);
        return jsonResponse([
            'success' => true
        ]);
    }

}
