<?php

namespace App\Http\Controllers;

use App\Models\Rank;
use App\Models\Transaction;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\DB;
use Shetabit\Multipay\Exceptions\InvalidPaymentException;
use Shetabit\Multipay\Invoice;
use Shetabit\Payment\Facade\Payment;

class RankController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth:sanctum');
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws Exception
     */
    public function transaction(Request $request): JsonResponse
    {
        $user = $request->user();
        $request->validate([
            'rank_id' => 'required'
        ]);
        $rank = Rank::query()->find($request->rank_id);
        if (!$rank) {
            return jsonResponse([
                'success' => false,
                'message' => 'پکیج مورد نظر یافت نشد.'
            ]);
        }
        if ($user->credit->value > $rank->getAttribute('credit_limit')) {
            DB::table('ranks_users')->insert([
                'rank_id' => $rank->id,
                'user_id' => $user->id
            ]);
            $user->credit()->decrement('value', $rank->getAttribute('credit_limit'));
            return jsonResponse([
                'success' => true,
                'link' => 'https://emza-group.ir/dashboard/rate'
            ]);
        }
        Cache::put($user->phone_number, $rank->id, 60 * 60);
        $amount = $rank->price - ($user->credit->value * 25000);
        return jsonResponse([
            'success' => true,
            'link' => $this->pay($amount / 10, $user)
        ]);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     */
    public function verify(Request $request): JsonResponse
    {
        $request->validate([
            'Authority' => 'required',
            'Status' => 'required'
        ]);
        $transaction = Transaction::query()->where('transaction_id', $request->Authority)->first();
        if (!$transaction) {
            return jsonResponse([
                'success' => false,
                'message' => 'تراکنش یافت نشد'
            ]);
        }
        $response = $this->verifyTxId($request->Authority, $transaction);
        if ($response['success']) {
            $user = $request->user();
            $rank = Rank::query()->find(Cache::get($user->phone_number));
            DB::table('ranks_users')->insert([
                'rank_id' => $rank->id,
                'user_id' => $user->id
            ]);
            Cache::forget($user->phone_number);
        }
        return jsonResponse($response);

    }

    /**
     * @throws Exception
     */
    protected function pay($amount, $user)
    {
        $invoice = new Invoice();
        $invoice->amount($amount);
        return Payment::callbackUrl("https://emza-group.ir/return_page")->purchase($invoice, function ($driver, $txId) use ($user, $amount) {
            $user->transactions()->create([
                'driver' => 'زرین پال',
                'amount' => $amount,
                'transaction_id' => $txId,
                'device' => true
            ]);
        })->pay()->getUrl();
    }

    /**
     * @param $txId
     * @param $transaction
     * @return array
     */
    protected function verifyTxId($txId, $transaction): array
    {
        try {
            $receipt = Payment::amount($transaction->amount)->transactionId($txId)->verify();
            $transaction->update([
                'ref_id' => $receipt->getReferenceId(),
                'status' => 'paid'
            ]);
            return [
                'success' => true,
                'message' => 'تراکنش موفق'
            ];
        } catch (InvalidPaymentException $e) {
            $transaction->update([
                'status' => 'failed'
            ]);
            return [
                'success' => false,
                'message' => $e->getMessage()
            ];
        } catch (Exception $e) {
            return [
                'success' => false,
                'message' => $e->getMessage()
            ];
        }
    }

    /**
     * @return JsonResponse
     */
    public function index(): JsonResponse
    {
        return jsonResponse([
            'success' => true,
            'ranks' => Rank::query()->get()
        ]);
    }

    /**
     * @param Request $request
     * @param $id
     * @return JsonResponse
     */
    public function update(Request $request, $id): JsonResponse
    {
        $data = $request->validate([
            'price' => 'required',
            'promotion' => 'sometimes',
            'credit_limit' => 'sometimes'
        ]);
        $rank = Rank::query()->find($id);
        if (!$rank) {
            return jsonResponse([
                'success' => false,
                'message' => 'پکیج مورد نظر یافت نشد'
            ]);
        }
        $rank->update($data);
        return jsonResponse([
            'success' => true,
            'rank' => $rank
        ]);
    }
}
