<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\HasOne;
use Illuminate\Database\Eloquent\Relations\MorphToMany;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Support\Facades\Storage;
use Laravel\Sanctum\HasApiTokens;

class Product extends Model
{
    use HasFactory, HasApiTokens, SoftDeletes;

    protected $guarded = [];

    protected $appends = ['rate_avg', 'is_cafe'];

    public function getIsCafeAttribute()
    {
        $category = Category::query()->find($this->category_id);
        return !is_null($category) ? $category->isCafe() : null;
    }

    public function isCafe()
    {
        return $this->getIsCafeAttribute();
    }

    public function getRate()
    {
        $this->getRateAvgAttribute();
    }

    public function getter()
    {
        $this->growthParts();
    }

    public function getRateAvgAttribute()
    {
       $rates = $this->rates()->get();
       $count = $rates->count();
       $sum = 0;
       foreach ($rates as $rate) {
           $sum += $rate->rate_amount;
       }
       if ($count > 0) return doubleval($sum / $count);
       else return 0;
    }
    /**
     * @param $value
     * @return string
     */
    public function getImageAttribute($value): string
    {
        return Storage::url($value);
    }

    public function category()
    {
        return $this->belongsTo(Category::class, 'category_id', 'id');
    }

    /**
     * @return MorphToMany
     */
    public function categories(): MorphToMany
    {
        return $this->morphToMany(Category::class, 'category');
    }

    /**
     * @return HasOne
     */
    public function event(): HasOne
    {
        return $this->hasOne(Event::class, 'product_id');
    }

    /**
     * @return BelongsToMany
     */
    public function sizes(): BelongsToMany
    {
        return $this->belongsToMany(Size::class, 'product_size')->withPivot('price');
    }

    /**
     * @param $product_id
     * @param $size_id
     * @param $price
     */
    public function updatePivot($product_id, $size_id, $price)
    {
        $pivot = ProductSize::query()
            ->where('product_id', $product_id)
            ->where('size_id', $size_id)
            ->whereNull('deleted_at')
            ->first();
        $pivot->update([
            'size_id' => $size_id,
            'price' => $price
        ]);
    }

    /**
     * @return BelongsToMany
     */
    public function branches(): BelongsToMany
    {
        return $this->belongsToMany(Branch::class, 'branch_product');
    }

    /**
     * @return HasMany
     */
    public function rates(): HasMany
    {
        return $this->hasMany(Rate::class, 'product_id');
    }

    public function growthParts(): BelongsToMany
    {
        return $this->belongsToMany(GrowthPart::class, 'product_metas', 'product_id', 'filter_id');
    }

    public function coffeeTypes(): BelongsToMany
    {
        return $this->belongsToMany(CoffeeType::class, 'product_metas', 'product_id', 'filter_id');
    }

    public function beanTypes(): BelongsToMany
    {
        return $this->belongsToMany(BeanType::class, 'product_metas', 'product_id', 'filter_id');
    }

    public function millTypes(): BelongsToMany
    {
        return $this->belongsToMany(MillType::class, 'product_metas', 'product_id', 'filter_id');
    }

    public function footerProducts()
    {
        return $this->hasMany(FooterProduct::class, 'product_id');
    }
}
