<?php


namespace App\Services;


use App\Models\Address;
use App\Models\Branch;
use App\Models\Category;
use App\Models\DeliveryPrice;
use App\Models\Order;
use App\Models\Permission;
use App\Models\Product;
use App\Models\Promotion;
use App\Models\Sentence;
use App\Models\Settings;
use App\Models\Slider;
use App\Models\Tag;
use App\Models\Terms;
use App\Models\User;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Str;

class MainPage
{
    /**
     * @param Request $request
     * @return array
     */
    public function all(Request $request): array
    {
        $user = $request->user();
        $category = Category::query()->where('category_type', Product::class)->where('type', 'CAFE')->get();
        $sliders = Slider::all();
        $product = Product::all();
        $term = Terms::all()->last();
        if (!$term) {
            $term = [];
        }
        $settings = Settings::query()->get()->last() ?? [];
        foreach ($category as $item) {

            $item['product'] = branch()
                ->products()
                ->with('sizes')
                ->where('category_id', $item->id)
                ->whereNull('deleted_at')
                ->withPivot('capacity')
                ->get();
            foreach ($item['product'] as $something) {
                $something['is_available'] = $something->pivot->capacity > 0;
            }
        }

        $notification_count = $user->notifications()->where('active', true)->get()->count();

        return [
            'success' => true,
            'user' => $user,
            'profile' => $user->profile,
            'category' => $category,
            'slider' => $sliders,
            'product' => $product,
            'notification_count' => $notification_count,
            'term' => $term,
            'settings' => $settings,
        ];
    }

    protected $cart_branch = null;

    public function cart(Request $request)
    {
        $user = $request->user();
        $cartItems = $user->cartItems()->get();
        if ($cartItems->count() > 0) {
            $this->cart_branch = Branch::query()->find($cartItems->first()->getAttribute('branch_id'));
        }
        foreach ($cartItems as $item) {
            $item['product'] = Product::query()->where('name', $item->product)->first();
        }
        $totalPrice = 0;
        foreach ($cartItems as $item) {
            $totalPrice = $totalPrice + $item->price * $item->quantity;
        }

        $addresses = $user->addresses;

        $data = $request->validate([
            'promotion_code' => 'nullable|size:12'
        ]);
        if ($data['promotion_code'] != null || $data['promotion_code'] != '') {
            $promotion = Promotion::query()
                ->where('promotion_code', '=', $data['promotion_code'])
                ->whereNull('deleted_at')
                ->first();
            if (!$promotion) {
                return jsonResponse([
                    'success' => false,
                    'message' => 'این کد تخفیف موجود نیست.'
                ]);
            }
            $percent = $promotion->percent / 100;
            $limit = $promotion->max_promo_limit;
            if (($totalPrice * $percent) < $limit) {
                $totalPrice = $totalPrice - $totalPrice * $percent;
            } else {
                $totalPrice = $totalPrice - $limit;
            }
        }
        return [
            'success' => true,
            'user' => $user,
            'cart_items' => $cartItems,
            'total_price' => $totalPrice,
            'addresses' => $addresses,
            'promotion' => $promotion ?? 0,
            'send_price' => exists($this->cart_branch) ?
                $this->cart_branch->getAttribute('delivery_price') :
                branch()->getAttribute('delivery_price')
        ];

    }

    /**
     * @param Request $request
     * @return array
     */
    public function history(Request $request): array
    {
        $user = $request->user();
        $orders = Order::query()->where('user_id', $user->id)->whereHas('transaction', function ($q) {
            $q->where('status', 'paid');
        })->with('orderItems')->latest()->get();
        foreach ($orders as $order) {
            $order['promotion'] = 0;
            if (Promotion::query()->where('promotion_code', $order->promotion_code)->first()) {
                $order['promotion'] = Promotion::query()->where('promotion_code', $order->promotion_code)->first();
            }
        }

        return [
            'orders' => $orders,
        ];
    }

    /**
     * @return array
     */
    public function welcome(): array
    {
        $term = Terms::all()->last();
        if (!$term) {
            $term = [];
        }
        return [
            'success' => true,
            'term' => $term,
        ];
    }

    /**
     * @return array
     */
    public function sentences(): array
    {
        $category = Category::query()->where('category_type', Sentence::class)->get();
        foreach ($category as $item) {
            $item['sentences'] = Sentence::query()->where('category_id', $item->id)->get();
            $item['tags'] = Tag::query()->where('category_id', $item->id)->get();
        }
        $tag = Tag::all();
        foreach ($tag as $item) {
            $item['sentences'] = $item->sentences()->get();
        }
        return [
            'success' => true,
            'category' => $category,
            'tag' => $tag,
        ];
    }

    /**
     * @param $id
     * @return array
     */
    public function orders($id): array
    {
        $user = User::query()->find($id);
        if (!$user) {
            return [
                'success' => false,
                'message' => 'کاربر مورد نظر یافت نشد'
            ];
        }
        $orders = Order::query()->where('user_id', $user->getAttribute('id'))->get();
        foreach ($orders as $order) {
            $order['address'] = Address::query()->find($order->address_id);
            $order['user'] = $user;
        }
        return [
            'success' => true,
            'orders' => $orders
        ];
    }

    /**
     * @return JsonResponse
     * @throws Exception
     */
    public function dashboard(): JsonResponse
    {
        $thisYear = now()->year;
        $months = [
            'january' => 1,
            'february' => 2,
            'march' => 3,
            'april' => 4,
            'may' => 5,
            'june' => 6,
            'july' => 7,
            'august' => 8,
            'september' => 9,
            'october' => 10,
            'november' => 11,
            'december' => 12
        ];
        $users = [];
        foreach ($months as $monthName => $monthNumber) {
            $users[$monthName] = User::query()
                ->whereYear('created_at', $thisYear)
                ->whereMonth('created_at', '<=', $monthNumber)
                ->count();
        }

        $user_count = User::query()->count();
        $orders = Order::query()
            ->whereHas('transaction', function ($q) {
                $q->where('status', 'paid');
            })
            ->get();
        $income = 0;
        foreach ($orders as $order) {
            $income = $income + $order->total_price;
        }
        $orders_count = $orders->count();

        $baseOrder = Order::query()
            ->whereHas('transaction', function ($q) {
                $q->where('status', 'paid');
            })
            ->whereYear('created_at', '=', $thisYear);
        $allOrders = [];
        foreach ($months as $monthName => $monthNumber) {
            $allOrders[$monthName] = $baseOrder
                ->whereMonth('created_at', '<=', $monthNumber)
                ->sum('total_price');
        }
        $response = [
            'success' => true,
            'userCount' => $user_count,
            'orderCount' => $orders_count,
            'income' => $income,
            'users' => $users,
            'orders' => $allOrders
        ];
        return jsonResponse($response);
    }

    /**
     * @return JsonResponse
     */
    public function checkBoxes(): JsonResponse
    {
        $permissions = Permission::all();
        foreach ($permissions as $permission) {
            $permission['before'] = Str::before($permission->key, '-');
            $permission['after'] = Str::after($permission->key, '-');
        }
        $response = [
            'success' => true,
            'permissions' => $permissions
        ];
        return jsonResponse($response);
    }
}
